@@ -1,7 +1,7 @@
 from posixpath import dirname
 from textwrap import indent
 from typing import *
-from susc import SusFile
+from susc import File
 from susc.things import *
 from os import makedirs, path, write
 from susc import log
@@ -75,12 +75,12 @@ def type_to_amogus(type_: SusType, obj_types: Dict[str, str]) -> str:
         return f"new amogus.repr.{t_name}({type_.name})"
 
     name = type_.name
-    if name == "Int" and type_.args[0] > 6:
+    if name == "Int" and type_.args[0] > 4:
         name = "BigInteger"
 
     return f"new amogus.repr.{name}({', '.join([str(x) for x in type_.args] + [''])}{type_validators(type_)})"
 
-def write_output(root_file: SusFile, target_dir: str) -> None:
+def write_output(root_file: File, target_dir: str) -> None:
     proj_name = path.splitext(path.basename(root_file.path))[0]
     header = ("/* Generated by AMOGUS SUSC (https://github.com/amogus-api/susc)\n"
     f" * Project name: {proj_name}\n *\n"
